﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Hotline.Caching.Interfaces;
using Hotline.EventBus;
using Hotline.FlowEngine.Notifications;
using Hotline.FlowEngine.Workflows;
using Hotline.Orders;
using Hotline.Settings;
using Hotline.Share.Enums.FlowEngine;
using Hotline.Snapshot.Notifications;
using MediatR;
using XF.Domain.Exceptions;

namespace Hotline.Application.Handlers.Order
{
    /// <summary>
    /// 需求：坐席派给网格员的安全隐患工单若未推送成功超过4小时或者网格员超过4小时没回复，则自动流转到标注节点待标注列表
    /// </summary>
    public class GuiderSystemTimeoutHandler : INotificationHandler<GuiderSystemTimeOutBackNotification>
    {
        private readonly ISystemSettingCacheManager _systemSettingCacheManager;
        private readonly IOrderRepository _orderRepository;
        private readonly IWorkflowDomainService _workflowDomainService;

        public GuiderSystemTimeoutHandler(
            ISystemSettingCacheManager systemSettingCacheManager,
            IOrderRepository orderRepository,
            IWorkflowDomainService workflowDomainService
            )
        {
            _systemSettingCacheManager = systemSettingCacheManager;
            _orderRepository = orderRepository;
            _workflowDomainService = workflowDomainService;
        }

        /// <summary>Handles a notification</summary>
        /// <param name="notification">The notification</param>
        /// <param name="cancellationToken">Cancellation token</param>
        public async Task Handle(GuiderSystemTimeOutBackNotification notification, CancellationToken cancellationToken)
        {
            bool.TryParse(
                _systemSettingCacheManager.GetSetting(SettingConstants.Snapshot)?.SettingValue[0],
                out bool isSnapshotEnable);
            if (isSnapshotEnable)
            {
                var order = await _orderRepository.GetAsync(notification.OrderId, cancellationToken);
                if (order is null)
                    throw new UserFriendlyException("无效工单编号");

                var workflow = await _workflowDomainService.GetWorkflowAsync(order.WorkflowId, withDefine: true,
                    withSteps: true, cancellationToken: cancellationToken);
                var currentStep = workflow.Steps.FirstOrDefault(d => string.CompareOrdinal(d.Tag, TagDefaults.Wanggeyuan) == 0);
                if (currentStep is null)
                    throw new UserFriendlyException($"未查询到网格员节点, workflowId: {workflow.Id}");
                if(currentStep.Status == EWorkflowStepStatus.Handled)
                    throw new UserFriendlyException($"网格员节点已办理完, workflowId: {workflow.Id}");
                var markStepDefine = workflow.WorkflowDefinition.FindStepDefineByTag(d =>
                        string.Compare(d.Tag, TagDefaults.OrderMark, StringComparison.OrdinalIgnoreCase) == 0);
                if (markStepDefine is null)
                    throw new UserFriendlyException("未配置工单标记节点或未正确配置tag");


            }
        }
    }
}
